"""
Octarver - Octave Range Converter

Convert between musical note names and frequencies.
"""

from .__main__ import note2freq, freq2note, main

__version__ = "0.1.2"
__all__ = ["freq2note", "note2freq", "main"]

